from setuptools import setup, find_packages


setup(name='seeta-dataset',
      version='0.0.7',
      platforms='any',
      maintainer_email='seetaas@seetatech.com',
      description='SeeTaaS DataSet',
      packages=find_packages(),
      install_requires=[
      ],
      classifiers=[
          'Programming Language :: Python',
          'Operating System :: OS Independent',
      ])
